/* 
 *
 * linux/arch/arm/boot/compressed/head-s3c2400.S
 * 
 * Copyright (C) 2001 Samsung Electronics by chc, 010406 
 * 
 * S3C2400 specific tweaks.  This file is merged with head.S by the linker.
 *
 */
#include <linux/config.h>
#include <linux/linkage.h>
#include <asm/mach-types.h>

// watch dog timer 
#define pWTCON		0x15300000

// interrupt control register 
#define pINTMSK		0x14400008
#define pINTOFFSET	0x14400014

#if 1 /*def NO_ARMBOOT*/ /* this is all done in armboot GJ */
// clock controller 
#define LOCKTIME	0x14800000
#define MPLLCON		(LOCKTIME + 0x4)
#define UPLLCON		(LOCKTIME + 0x8)
#define CLKCON		(LOCKTIME + 0xc)
#define CLKDIVN		(LOCKTIME + 0x14)
//
//	if FCLK_SPEED is changed, modify the three files
//
// 	uncompress.h
// 	serial_s3c2400.c
//	time.h 
//
// condition: Fin=12Mhz, Fout=FCLK
//					clock divider (FCLK, HCLK, PCLK)	
// FCLK_SPEED	0	FCLK=50Mhz, 	1:1:1
// 		1	FCLK=66Mhz, 	1:1:1
// 		2	FCLK=133Mhz, 	1:2:4
// 		3	FCLK=33Mhz, 	1:1:1
// 		4	FCLK=80Mhz, 	1:4:4
// 		5	FCLK=200Mhz, 	1:2:4
//		6	FCLK=133Mhz,	1:2:2
#define FCLK_SPEED	2 /* was 1 - GJ */

// HPCLKMODE	0x0	1:1:1
//		0x1	1:1:2
//		0x2	1:2:2
//		0x3	1:2:4
//		0x8	1:4:4
#if FCLK_SPEED==0
#define HPCLKMODE	0x0
#elif FCLK_SPEED==1
#define HPCLKMODE	0x0
#elif FCLK_SPEED==2
#define HPCLKMODE	0x3
#elif FCLK_SPEED==3
#define HPCLKMODE	0x0
#elif FCLK_SPEED==4
#define HPCLKMODE	0x8
#elif FCLK_SPEED==5
#define HPCLKMODE	0x3
#elif FCLK_SPEED==6
#define HPCLKMODE	0x2
#endif

// memory controller
#define	BWSCON		0x14000000
#define	BANKCON3	(BWSCON + 0x10)	/* for cs8900, ethernet */
#define REFRESH		(BWSCON + 0x24)

// pre-defined constants
#define USERMODE	0x10
#define	FIQMODE		0x11
#define IRQMODE		0x12
#define SVCMODE		0x13
#define ABORTMODE	0x17
#define UNDEFMODE	0x1b
#define MODEMASK	0x1f
#define NOINT		0xc0


#if FCLK_SPEED==0
#define M_MDIV		0x5c
#define M_PDIV		0x4
#define M_SDIV		0x2
#elif FCLK_SPEED==1
#define M_MDIV		0x3a
#define M_PDIV		0x4
#define M_SDIV		0x1
#elif FCLK_SPEED==2
#define M_MDIV		0x7d
#define M_PDIV		0x4
#define M_SDIV		0x1
#elif FCLK_SPEED==3
#define M_MDIV		0x7c
#define M_PDIV		0x4
#define M_SDIV		0x3
#elif FCLK_SPEED==4
#define M_MDIV		0x98
#define M_PDIV		0x4
#define M_SDIV		0x2
#elif FCLK_SPEED==5
#define M_MDIV		0x5c
#define M_PDIV		0x4
#define M_SDIV		0x0
#elif FCLK_SPEED==6
#define M_MDIV		0x7d
#define M_PDIV		0x4
#define M_SDIV		0x1
#endif

//	FCLK(Mhz)	M	P	S
//////////////////////////////////////////
//	33		0x7c	4	3
//	50		0x5c	4	2	// default
//	66		0x3a	4	1
//	70		0x3e	4	1
//	75		0x8e	4	2
//	75		0x43	4	1
//	80		0x98	4	2
//////////////////////////////////////////
//	100		0x5c	4	1
//	133		0x7d	4	1
//	140		0x84	4	1
//	150		0x8e	4	1
//////////////////////////////////////////
//	170		0xa2	4	1
//	180		0x52	4	0
//	200		0x5c	1	1	// old... what ? ask snoopy...
//	200		0x5c	4	0
//	220		0x66	4	0

// Bank0
#define B0_Tacs	0x0	// 0 clk
#define B0_Tcos	0x0	// 0 clk
#define B0_Tacc	0x7	// 14 clk
#define B0_Tcoh	0x0	// 0 clk
#define B0_Tah	0x0	// 0 clk
#define B0_Tacp	0x0
#define B0_PMC	0x0	// normal
 
// Bank1
#define B1_Tacs	0x0	// 0 clk
#define B1_Tcos	0x0	// 0 clk
#define B1_Tacc	0x7	// 14 clk
#define B1_Tcoh	0x0	// 0 clk
#define B1_Tah	0x0	// 0 clk
#define B1_Tacp	0x0
#define B1_PMC	0x0	// normal
 
// Bank2
#define B2_Tacs	0x0	// 0 clk
#define B2_Tcos	0x0	// 0 clk
#define B2_Tacc	0x7	// 14 clk
#define B2_Tcoh	0x0	// 0 clk
#define B2_Tah	0x0	// 0 clk
#define B2_Tacp	0x0
#define B2_PMC	0x0	// normal
 
// Bank3
#define B3_Tacs	0x0	// 0 clk
#define B3_Tcos	0x0	// 0 clk
#define B3_Tacc	0x7	// 14 clk
#define B3_Tcoh	0x0	// 0 clk
#define B3_Tah	0x0	// 0 clk
#define B3_Tacp	0x0
#define B3_PMC	0x0	// normal
 
// Bank4
#define B4_Tacs	0x0	// 0 clk
#define B4_Tcos	0x0	// 0 clk
#define B4_Tacc	0x7	// 14 clk
#define B4_Tcoh	0x0	// 0 clk
#define B4_Tah	0x0	// 0 clk
#define B4_Tacp	0x0
#define B4_PMC	0x0	// normal
 
// Bank5
#define B5_Tacs	0x0	// 0 clk
#define B5_Tcos	0x0	// 0 clk
#define B5_Tacc	0x7	// 14 clk
#define B5_Tcoh	0x0	// 0 clk
#define B5_Tah	0x0	// 0 clk
#define B5_Tacp	0x0
#define B5_PMC	0x0	// normal
 
// Bank6
#define	B6_MT	0x3	// SDRAM 
#define	B6_Trcd	0x1	// 3clk
#define	B6_SCAN	0x1	// 9 bit

// Bank7
#define	B7_MT	0x3	// SDRAM 
#define	B7_Trcd	0x1	// 3clk
#define	B7_SCAN	0x1	// 9 bit

// refresh parameter
#define REFEN	0x1	// enable refresh
#define TREFMD	0x0	// CBR(CAS before RAS)/auto refresh
#define Trp	0x0	// 2 clk
#define Trc	0x3	// 7 clk
#define Tchr	0x2 	// 3 clk

#if FCLK_SPEED==0
#define REFCNT	1269	// period=15.6 us, HCLK=50Mhz, (2048+1-15.6*50)
#elif FCLK_SPEED==1
#define REFCNT	1017	// period=15.6 us, HCLK=66Mhz, (2048+1-15.6*66)
#elif FCLK_SPEED==2
#define REFCNT	1011	// period=15.6 us, HCLK=66.5Mhz, (2048+1-15.6*66.5)
#elif FCLK_SPEED==3
#define REFCNT	1534	// period=15.6 us, HCLK=33Mhz, (2048+1-15.6*33)
#elif FCLK_SPEED==4
#define REFCNT	1737	// period=15.6 us, HCLK=20Mhz, (2048+1-15.6*20)
#elif FCLK_SPEED==5
#define REFCNT	489	// period=15.6 us, HCLK=100Mhz, (2048+1-15.6*100)
#elif FCLK_SPEED==6
#define REFCNT	1011	// period=15.6 us, HCLK=66.5Mhz, (2048+1-15.6*66.5)
#endif
#endif /* NO_ARMBOOT */


// time to eat

//	.section	".start", #alloc, #execinstr
	.section	".start", "ax"

__s3c2400_start:


	//ldr	r0, =pWTCON	
	mov	r0, #0x15000000
	add     r0, r0, #0x00300000
	//ldr	r1, =0x0	// watchdog disabled
	mov	r1, #0x0	// watchdog disabled
	str	r1, [r0]

	//ldr	r0, =pINTMSK
	mov	r0, #0x14000000
	add     r0, r0, #0x00400000
	add     r0, r0, #0x00000008     
	//ldr	r1, =0xffffffff	// all interrupts disabled
	mov	r1, #0xffffffff	// all interrupts disabled
	str	r1, [r0]


// PIC (Position Independent Code) 
// Check the processor works correctly by LED on
                mov	r0, #0x15000000         @ 0x15600014     
                add     r0, r0, #0x00600000
                add     r0, r0, #0x00000014     
                mov     r1, 	#0x000000ff	@ PC0~3: reserved (0x11)
                add	r1, r1, #0x0000ff00	@ PC4~7: reserved (0x11)
                add	r1, r1, #0x00ff0000	@ PC8~11: reserved (0x11)
                add	r1, r1, #0x55000000	@ PC12~15: output (0x01)
                str     r1, [r0]

                mov	r0, #0x15000000                 @ 0x15600018     
                add     r0, r0, #0x00600000
                add     r0, r0, #0x00000018     
                mov     r1, #0xaf00			@ 1010 -> D2:D3:D4:D5 = off:on:off:on ( 1:off, 0:on )
                add     r1, r1, #0x00ff
                str     r1, [r0]


#if 1 /*def NO_ARMBOOT*/ /* this is all done in armboot GJ */
	ldr	r0, =CLKDIVN	
	ldr	r1, =HPCLKMODE
	str	r1, [r0]

// to reduce PLL lock time, adjust the LOCKTIME register
	ldr	r0, =LOCKTIME
	ldr	r1, =0xffffff
	str	r1, [r0]

// PLL configuration
	ldr	r0, =MPLLCON
	ldr	r1, =((M_MDIV<<12) + (M_PDIV<<4) + M_SDIV) // Fin=12Mhz, Fout=50Mhz
	str	r1, [r0]

// memory control configuration
// PIC(Position Independent Code) code. It's only 'ldr r0, =SMRDATA'
REF:	ldr	r0, =REF
	ldr	r1, =SMRDATA
	sub	r0, r1, r0	// r0 = SMRDATA offset from REF
	add	r0, r0, pc
	sub	r0, r0, #20	// r0 = SMRDATA offset from here, REF2	
REF2:	nop	

	ldr	r1, =BWSCON	// Bus Width Status Controller

	add	r2, r0, #52	// end address of SMRDATA
0:
	ldr	r3, [r0], #4
	str	r3, [r1], #4
	cmp	r2, r0
	bne	0b

#endif /* NO_ARMBOOT */
	
jumptoram:
#if 1 /*def NO_ARMBOOT*/ /* this is all done in armboot GJ */
// only for cs8900, GCS3 (0x0600 0000 ~ 0x0800 0000)
	ldr	r0, =BWSCON
	ldr	r1, [r0]	
	ldr	r2, =0xf000
	bic	r1, r1, r2	/* clear Bank 3 */
	ldr	r3, =0xd000	/* Using UB/LB, WAIT enable, 16 data bus width for bank 3 */
	orr	r1, r1, r3	
	str	r1, [r0]

	ldr	r0, =BANKCON3	/* cs8900 bank 3 */
	ldr	r1, [r0]
	ldr	r2, =0xffff
	bic	r1, r1, r2	/* clear bank control 3 */
	ldr	r3, =0x1f4c	/* Tacs=00b (0 clock), Tcos=11b (4 clock), Tacc=111b (14 clocks), Toch=01b (1 clock), Tcah=00b (0 clock), Tacp=11b (6 clocks), PMC=00b (normal 1 data) */
	orr	r1, r1, r3
	str	r1, [r0]
#endif /* NO_ARMBOOT */


// stack configuration is omitted because it is reconfigured in entry-armv.S

// IRQ handler configuratino is omitted because reconfigured in entry-armv.S

// Enable Interrupt in CPSR by chc, advised by prunnamu
	mrs	lr, cpsr
	bic	lr, lr, #0x80 // enable IRQ interrupt in CPSR
	msr	cpsr, lr
	
	b	1f

#if 1 /*def NO_ARMBOOT*/ /* this is all done in armboot GJ */
	.ltorg
// the literal pools origin

SMRDATA:
	.word	0x2222d220	// d->Ethernet, BUSWIDTH=32
	.word	((B0_Tacs<<13)+(B0_Tcos<<11)+(B0_Tacc<<8)+(B0_Tcoh<<6)+(B0_Tah<<4)+(B0_Tacp<<2)+(B0_PMC)) // GCS0
	.word	((B1_Tacs<<13)+(B1_Tcos<<11)+(B1_Tacc<<8)+(B1_Tcoh<<6)+(B1_Tah<<4)+(B1_Tacp<<2)+(B1_PMC)) // GCS1
	.word	((B2_Tacs<<13)+(B2_Tcos<<11)+(B2_Tacc<<8)+(B2_Tcoh<<6)+(B2_Tah<<4)+(B2_Tacp<<2)+(B2_PMC)) // GCS2
	.word	((B3_Tacs<<13)+(B3_Tcos<<11)+(B3_Tacc<<8)+(B3_Tcoh<<6)+(B3_Tah<<4)+(B3_Tacp<<2)+(B3_PMC)) // GCS3
	.word	((B4_Tacs<<13)+(B4_Tcos<<11)+(B4_Tacc<<8)+(B4_Tcoh<<6)+(B4_Tah<<4)+(B4_Tacp<<2)+(B4_PMC)) // GCS4
	.word	((B5_Tacs<<13)+(B5_Tcos<<11)+(B5_Tacc<<8)+(B5_Tcoh<<6)+(B5_Tah<<4)+(B5_Tacp<<2)+(B5_PMC)) // GCS5
	.word	((B6_MT<<15)+(B6_Trcd<<2)+(B6_SCAN)) // GCS6
	.word	((B7_MT<<15)+(B7_Trcd<<2)+(B7_SCAN)) // GCS7
	.word	((REFEN<<23)+(TREFMD<<22)+(Trp<<20)+(Trc<<18)+(Tchr<<16)+REFCNT)
	.word	0x10	// BUSWIDTH=32, SCLK power saving mode, BANKSIZE 32M/32M

	.word	0x30	// MRSR6, CL=3clk
	.word	0x30	// MRSR7
	
	.align
#endif /* NO_ARMBOOT */

1:
	mov	r7, #MACH_TYPE_SMDK2400
	mov	r8, #0
	@ Data cache might be active.
	@ Be sure to flush kernel binary out of the cache,
	@ whatever state it is, before it is turned off.
	@ This is done by fetching through currently executed
	@ memory to be sure we hit the same cache.
	@ stolen from head-sa1100.S
	bic	r2, pc, #0x1f
	add	r3, r2, #0x4000		@ 16 kb is quite enough...
20:	ldr	r0, [r2], #32
	teq	r2, r3
	bne	20b
	mcr	p15, 0, r0, c7, c10, 4	@ drain WB
	mcr	p15, 0, r0, c7, c7, 0	@ flush I & D caches

	@ disabling MMU and caches
	mrc	p15, 0, r0, c1, c0, 0	@ read control reg
	bic	r0, r0, #0x0d		@ clear WB, DC, MMU
	bic	r0, r0, #0x1000		@ clear Icache
	mcr	p15, 0, r0, c1, c0, 0
