/*
 * SMDK2400 I2C interface.
 * Copyright (c) 2001 DENX Software Engineering, Wolfgang Denk, wd@denx.de
 *
 * SMDK2400 specific parts of the I2C interface
 */

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/stddef.h>
#include <linux/sched.h>

#include <linux/i2c.h>
#include <linux/i2c-algo-s3c2400x.h>

static void
smdk2400_i2c_init(struct i2c_algo_s3c2400x_data *data)
{
}

static int smdk2400_reg(struct i2c_client *client)
{
	return 0;
}

static int smdk2400_unreg(struct i2c_client *client)
{
	return 0;
}

static void smdk2400_inc_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

static void smdk2400_dec_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

static struct i2c_algo_s3c2400x_data smdk2400_data = {
	udelay: 5,
	retry: 2000,
	int_sleep: 2*HZ/100
};


static struct i2c_adapter smdk2400_ops = {
	"smdk2400",
	I2C_HW_S3C2400X_SMDK2400,
	NULL,
	&smdk2400_data,
	smdk2400_inc_use,
	smdk2400_dec_use,
	smdk2400_reg,
	smdk2400_unreg,
};

int __init i2c_smdk2400_init(void)
{
	printk("i2c-smdk2400.o: i2c SMDK2400 module\n");

	/* reset hardware to sane state */
	smdk2400_i2c_init(&smdk2400_data);

	if (i2c_s3c2400x_add_bus(&smdk2400_ops) < 0) {
		printk("i2c-smdk2400.o: Unable to register with I2C\n");
		return -ENODEV;
	}

	return 0;
}

void __exit i2c_smdk2400_exit(void)
{
	i2c_s3c2400x_del_bus(&smdk2400_ops);
}

#ifdef MODULE
MODULE_AUTHOR("Wolfgang Denk <wd@denx.de>");
MODULE_DESCRIPTION("I2C-Bus adapter routines for smdk2400");
#endif

module_init(i2c_smdk2400_init);
module_exit(i2c_smdk2400_exit);
