/*
 * linux/include/asm-arm/arch-s3c2400/hardware.h
 *
 * Copyright (C) 1998 Nicolas Pitre <nico@visuaide.com>
 *
 * This file contains the hardware definitions for SA1100 architecture
 */

#ifndef __ASM_ARCH_HARDWARE_H
#define __ASM_ARCH_HARDWARE_H

/*
 * RAM definitions
 */
#define KERNTOPHYS(a)           ((unsigned long)(&a))

/* Flushing areas */
#if 0 /* I have no idea what this stuff is - GJ */
#define FLUSH_BASE_PHYS		0xe0000000
#define FLUSH_BASE		0xdf000000
#define FLUSH_BASE_MINICACHE	0xde000000
#endif

/*
 * S3C2400 internal I/O mappings
 *
 * We have the following mapping:
 * 	phys		virt
 * 	14000000	f4000000
 * 	15000000	f5000000
 *
 */

/* virtual start of IO space */
#define VIO_BASE	0xf4000000

/* x = IO space shrink power i.e. size = space/(2^x) */
#define VIO_SHIFT	3

/* virtual end of IO space */
#define VIO_END		0xf5ffffff	
#define VIO_SIZE	(VIO_END - VIO_BASE)

/* physical start of IO space */
#define PIO_START	0x14000000

#define io_p2v( x ) ((x) + 0xe0000000)

#define io_v2p( x ) ((x) - 0xe0000000)

/*
 * S3C2400 flash memory
 *
 * There are 2 flash banks on the SMDK2400 board:
 *
 * bank	type			phys addr	bank size	flash size
 * 0	2x Intel 28F640J3A	0x00000000	32Mb		16Mb
 * 1	1x AMD AM29LV800B	0x02000000	32Mb		1Mb
 *
 * Note, the flash banks may swap depending on the state
 * of the J4/J5 jumpers.
 */

#define FLASH0_BASE	0x00000000
#define FLASH0_SIZE	(32 << 20)
#define FLASH1_BASE	0x02000000
#define FLASH1_SIZE	(32 << 20)

#ifdef __ASSEMBLY__
#define LANGUAGE Assembly
#else
#define LANGUAGE C
#endif

/*
 * S3C2400 internal IO definitions
 */

#include "s3c2400.h"

/* #define PCIO_BASE	0xe0000000 */	/* PCMCIA0 IO space */

#endif
