/*
 * include/asm-arm/arch-s3c2400/irq.h
 *
 * Copyright (C) 1996-1998 Russell King
 * Copyright (C) 1999 Hugo Fiennes
 * Copyright (C) 2001 HyukChin Chang for S3C2400
 *
 * Changelog:
 *   22-08-1998	RMK	Restructured IRQ routines
 *   06-01-1999 HBF     SA1100 twiddles
 *   12-02-1999 NP	added ICCR
 *   17-02-1999 NP	empeg henry ugly hacks now in a separate file ;)
 *   11-08-1999 PD      SA1101 support added
 */

#ifndef __ASM_ARCH_HARDWARE_H
#include <asm/arch/hardware.h>
#endif

#define fixup_irq(x) 	(x)
static void s3c2400_mask_and_ack_irq(unsigned int irq)
{
    	// by chc
	// ack is done in interrupt service routine, by chc
    	rINTMSK |= ((unsigned long)1 << irq);
	// if the following registers are masked, then the clock(BogoMIPS) cannoe be executed
	rSRCPND = ((unsigned long)1 << irq);
	rINTPND = ((unsigned long)1 << irq);
}

static void s3c2400_mask_irq(unsigned int irq)
{
	// by chc
    	rINTMSK |= ((unsigned long) 1 << irq);
}


static void s3c2400_unmask_irq(unsigned int irq)
{
    	// by chc
    	rINTMSK &= ~((unsigned long)1 << irq);
}

extern void PRINTK(char *fmt, ...);
static __inline__ void irq_init_irq(void)
{
	unsigned long flags;
	int irq;

	save_flags_cli (flags);

	// by chc
        /* Disable all IRQs */
	rINTMSK = 0xffffffff; /* all masked */
	rSRCPND = 0xffffffff;
	rINTPND = 0xffffffff;

	/* All IRQs are IRQ, not FIQ */
	rINTMOD = 0x00000000; /* all IRQ mode */

	/* Turn off priority rotation */
	rPRIORITY = 0;

	restore_flags (flags);

	for (irq = 0; irq < NR_IRQS; irq++) {
		irq_desc[irq].valid	= 1;
		irq_desc[irq].probe_ok	= 1;
		irq_desc[irq].mask_ack	= s3c2400_mask_and_ack_irq;
		irq_desc[irq].mask	= s3c2400_mask_irq;
		irq_desc[irq].unmask	= s3c2400_unmask_irq;
	}
}

/*
 * This prototype is required for cascading of multiplexed interrupts.
 * Since it doesn't exist elsewhere, we'll put it here for now.
 */
extern void do_IRQ(int irq, struct pt_regs *regs);
