/*
 * linux/include/asm-arm/arch-s3c2400/time.h
 *
 * Copyright (C) 1998 Deborah Wallach.
 * Copyright (C) 2001 HyukChin Chang for S3C2400
 *
 */

#include <asm/arch/hardware.h>
#include <asm/arch/irqs.h>

/* NB: moved s3c2400_get_rtc_time(void) and s3c2400_set_rtc(void) to */
/* s3c2400-rtc.c - GJ Sep 8 2002 */
extern unsigned long s3c2400_get_rtc_time(void);
int s3c2400_set_rtc(void);

static unsigned long s3c2400_gettimeoffset (void)
{
/* by chc */
/* for temporal, check it out !!!  */
/* it is used at rand_initialize */
	unsigned long offset = 0;

	offset = tick;
	offset +=  rTCNTB4 * tick / LATCH;
	return offset;
}

/*
 * Reset the timer match every time to get centisecond interrutps
 */
extern __inline__ int s3c2400_reset_timer (void)
{
  long flags;
  /* by chc 01/04/10 */
  /* TCNTB4 are loaded into TCNT4 when the timer reached 0 automatically. When TCNT4 reaches 0, the interrupt reques will occur if the interrupt is enabled */
/* automatically reload in S3C2400 by chc */
  
  save_flags_cli( flags );
  restore_flags( flags );
  return(1);
}

/*
 * timer_interrupt() needs to keep up the real-time clock,
 * as well as call the "do_timer()" routine every clocktick.
 */
extern void PRINTK(char *fmt, ...);
/* lie about the argument type since we only pass a NULL */
extern void serial_s3c2400_check_rx_overrun(void *info);
static void s3c2400_timer_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	if (s3c2400_reset_timer ())
		do_timer(regs);
	/* for ethernet by chc */
	*(volatile unsigned short *)(0xe7000300 +  0x000a) = 0x0116;
	*(volatile unsigned short *)(0xe7000300 +  0x000c) = 0x0000;
	*(volatile unsigned short *)(0xe7000300 +  0x000a) = 0x0116;
	*(volatile unsigned short *)(0xe7000300 +  0x000c) = 0x8000;
	/* hack for serial rx - GJ */
	serial_s3c2400_check_rx_overrun(NULL);

}

static inline void setup_timer (void)
{
	gettimeoffset = s3c2400_gettimeoffset;
	set_rtc = s3c2400_set_rtc;
	xtime.tv_sec = s3c2400_get_rtc_time();
	timer_irq.handler = s3c2400_timer_interrupt;
#ifdef NO_ARMBOOT /* this is all done in armboot GJ */
	rTCFG0 = 0x0f00; /* prescaler value = 15 */
	/* 115200 */
#if 0	/* PCLK */
	rTCNTB4 = 6250; /* 20, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
	rTCNTB4 = 5156; /* 16.5, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
	rTCNTB4 = 28125; /* 90, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
	rTCNTB4 = 23237; /* 75, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
	rTCNTB4 = 15626; /* 50, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
	rTCNTB4 = 31250; /* 100, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
	rTCNTB4 = 25000; /* 80, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
	rTCNTB4 = 21875; /* 70, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
	rTCNTB4 = 10312; /* 33, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
#endif
	rTCNTB4 = 20626; /* 66, be cautious, TCNTB4 be below 2^16=65535 [15:0] */
			/* period = (prescaler value + 1)*(divider value)*buffer count / PCLK = 10 ms */
       			/* 10 ms = (15+1) * 2 * 15625 / (50 * 1000000) */

			/* 15626=10 ms * ( 50 * 1,000,000) / 2 / (15+1) */
			/* 20625=10 ms * ( 66 * 1,000,000) / 2 / (15+1) */
			/* 10312=10 ms * ( 33 * 1,000,000) / 2 / (15+1) */
			/* 6250=10 ms * ( 20 * 1,000,000) / 2 / (15+1) */
			/* 5156=10 ms * ( 16.5 * 1,000,000) / 2 / (15+1) */
			/* 31250=10 ms * ( 100 * 1,000,000) / 2 / (15+1) */
			/* 28125=10 ms * ( 90 * 1,000,000) / 2 / (15+1) */
			/* 25000=10 ms * ( 80 * 1,000,000) / 2 / (15+1) */
			/* 23437=10 ms * ( 75 * 1,000,000) / 2 / (15+1) */
			/* 21875=10 ms * ( 70 * 1,000,000) / 2 / (15+1) */
			
  	s3c2400_reset_timer();
	
	rTCON = 0x600000; /* interval mode(auto reload), update TCNTB4, stop */
#endif /* NO_ARMBOOT */
	setup_arm_irq(IRQ_TIMER4, &timer_irq);
#ifdef NO_ARMBOOT /* this is all done in armboot GJ */
	rTCON = 0x500000; /* interval mode, no operation, start for Timer 4 */
#endif /* NO_ARMBOOT */
}

