/*
 * linux/include/linux/tsc2000.c
 */

#ifndef _LINUX_TSC2000_H_
#define _LINUX_TSC2000_H_

#include <linux/types.h>
#include <linux/ioctl.h>

#define TSC2000_MAJOR			243
#define TSC2000_MINOR_BATTERY		1
#define TSC2000_MINOR_PRESSURE 		2
#define TSC2000_MINOR_TEMP0		3
#define TSC2000_MINOR_TEMP1		4
#define TSC2000_MINOR_TEMP2		5
#define TSC2000_MINOR_TEMP3		6
#define TSC2000_MINOR_TEMP4		7
#define TSC2000_MINOR_TEMP5		8
#define TSC2000_MINOR_TEMP6		9
#define TSC2000_MINOR_TEMP7		10
#define TSC2000_MINOR_TEMP8		11
#define TSC2000_MINOR_TEMP9		12
#define TSC2000_MINOR_TEMP10		13
#define TSC2000_MINOR_CONTACT_TEMP	14
#define TSC2000_MINOR_BRIGHTNESS	15

#define TSC2000_NAME			"tsc2000"

#define TSC2000_IOCTL_SET_MIN		_IOW('a', 0, tsc2000_data_t)
#define TSC2000_IOCTL_SET_MAX		_IOW('a', 1, tsc2000_data_t)
#define TSC2000_IOCTL_SET_DELTA		_IOW('a', 2, tsc2000_data_t)
#define TSC2000_IOCTL_SET_INTERVAL	_IOW('a', 3, tsc2000_data_t)
#define TSC2000_IOCTL_GET_VALUE		_IOR('a', 4, tsc2000_data_t)
#define TSC2000_IOCTL_GET_STATUS	_IOR('a', 5, tsc2000_data_t)
#define TSC2000_IOCTL_SET_BRIGHTNESS	_IOW('a', 6, tsc2000_data_t)
#define TSC2000_IOCTL_SET_CORRECTION	_IOW('a', 7, tsc2000_correction_t)
#define TSC2000_IOCTL_GET_CORRECTION	_IOR('a', 8, tsc2000_correction_t)

#define TSC2000_STATUS_MIN		0x01
#define TSC2000_STATUS_MAX		0x02
#define TSC2000_STATUS_DELTA		0x04

typedef struct tsc2000_data_s {
	__u32 value;
} tsc2000_data_t;

/* The structure describing the correction factors
 * used in calculations. Can only be set.
 */
typedef struct tsc2000_correction_s {
	/* abberration of the reference voltage compared to nominal, ppm */
	__s32 error_vref;
	/* abberration of the amplification factor compared to nominal, ppm */
	__s32 error_amp1;	
	__s32 error_amp2;
	__s32 error_amp3;
	/* offset of the pt1000 reference bridge, uV */
	__s32 offset_pt1000;	
	/* Correction factor for the battery voltage, o/oo */
	__s32 error_battery;
} tsc2000_correction_t;

#endif /* _LINUX_TSC2000_H_ */
